proc tac_add_buddy {name group} {
    if {[catch {set t $::BUDDIES($name,online)}] != 0} {
        set ::BUDDIES($name,online) F
        set ::BUDDIES($name,name) $name
        set ::BUDDIES($name,idle) 0
        toc_add_buddy $::SCREENNAME $name
    }

    if {[catch {set t $::GROUPS($group,people)}] != 0} {
        set ::GROUPS($group,people) [list]
        set ::GROUPS($group,collapsed) F
        lappend ::BUDDYLIST $group
        lappend ::GROUPS($group,people) $name
    } else {
        lappend ::GROUPS($group,people) $name
    }
}

proc tac_remove_buddy {buddy group} {
    set li [lsearch $::GROUPS($group,people) [normalize $buddy]]
    if {$li != -1} {
        set ::GROUPS($group,people) [lreplace $::GROUPS($group,people) $li $li]

        foreach i $::BUDDYLIST {
            lappend total $i
            lappend total $::GROUPS($i,people)
        }
        if {[lsearch $total $buddy] == -1} {
            catch { unset ::BUDDIES($buddy,online) }
            catch { unset ::BUDDIES($buddy,name) }
            catch { unset ::BUDDIES($buddy,idle) }
            catch { unset ::BUDDIES($buddy,evil) }
            catch { unset ::BUDDIES($buddy,signon) }
        }
        tac_set_config
    }
}

proc tac_remove_group {group} {
    if {[lsearch $::BUDDYLIST $group] != -1} {
        set li [lsearch -exact $::BUDDYLIST $group]
        if {$li != -1} {
            set ::BUDDYLIST [lreplace $::BUDDYLIST $li $li]
        }
        tac_set_config
    }
}

proc tac_add_group {group} {
    if {[catch {set t $::GROUPS($group,people)}] != 0} {
        set ::GROUPS($group,people) [list]
        set ::GROUPS($group,collapsed) F
        lappend ::BUDDYLIST $group
        tac_set_config
    }
}

proc tac_cmd_add {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([a-zA-Z0-9]+) (.*)$} $line match cmd buddy group]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    tac_add_buddy $buddy $group
    tac_set_config
}

proc tac_cmd_remove {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([a-zA-Z0-9]+) (.*)$} $line match cmd buddy group]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    tac_remove_buddy $buddy $group
}

proc tac_cmd_rmgroup {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([a-zA-Z0-9]+) *$} $line match cmd group]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    tac_remove_group $group
}

proc tac_cmd_addgroup {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([a-zA-Z0-9]+) *$} $line match cmd group]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    tac_add_group $group
}

proc tac_cmd_lsgroup {exp line} {
    foreach i $::BUDDYLIST {
        tac_output $i
    }
}


proc tac_cmd_list {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) +([a-zA-Z0-9]+) *$} $line match cmd group]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    if {[lsearch $::BUDDYLIST $group] != -1} {
        foreach i $::GROUPS($group,people) {
            tac_output $i
        }
    }
}

proc tac_cmd_listall {exp line} {
    foreach i $::BUDDYLIST {
        tac_output $i
        foreach j $::GROUPS($i,people) {
            tac_output "  $j"
        }
    }
}

proc tac_cmd_export {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) (.*)$} $line match cmd filename]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    set f1 [open $filename w]
    puts $f1 "m $::PDMODE"
    foreach i $::BUDDYLIST {
        puts $f1 "g $i"
        foreach j $::GROUPS($i,people) {
            puts $f1 "b $j"
        } 
    }
    close $f1
}

proc tac_cmd_import {exp line} {
    if {![regexp -- {^(/[a-zA-Z0-9]+) (.*)$} $line match cmd filename]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    tac_import_config $filename
}

tac_register_cmd "^/ba "      tac_cmd_add            "  /ba buddy group  add a buddy to the specified group"
tac_register_cmd "^/br "      tac_cmd_remove         "  /br buddy group  remove a buddy from your list"
tac_register_cmd "^/bl "      tac_cmd_list           "  /bl {group}      list buddies on your list"
tac_register_cmd "^/bl$"      tac_cmd_listall
tac_register_cmd "^/gl$"      tac_cmd_lsgroup        "  /gl              list all the groups"
tac_register_cmd "^/ga "      tac_cmd_addgroup       "  /ga group        add an empty group"
tac_register_cmd "^/gr "      tac_cmd_rmgroup        "  /gr group        remove a group and all buddies inside"
tac_register_cmd "^/bx "      tac_cmd_export         "  /bx file         export config list to file"
tac_register_cmd "^/bi "      tac_cmd_import         "  /bi file         import config list from file"
